/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.p4java.extension.operation;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;
import java.util.Map;

import com.perforce.p4java.extension.command.PerforceDomain;
import com.perforce.p4java.extension.command.RequestImpl;
import com.perforce.p4java.extension.exception.ServerAccessException;
import com.perforce.p4java.extension.exception.ServerConnectionException;
import com.perforce.p4java.extension.exception.ServerErrorException;
import com.perforce.p4java.extension.path.PathStringFormat;
import com.perforce.p4java.extension.path.PerforcePath;
import com.perforce.p4java.extension.server.Server;

public class FstatCommand {

    public static String tagDigest = "digest";
    public static String tagAttrPrefix = "attr-";
    public static String tagHeadRevision = "headRev";

    // output everything from all revisions
    private static String allRevsFlags = "-Oalf";
    // output everything including digest (al), nothing client related (s)
    private static String oneRevFlags = "-Oals";

    public static Map<String, Object>[] getFstats(final PerforcePath path,
	    final boolean allRevisions, final Server server)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	return server.execute(new RequestImpl() {

	    @Override
	    public String getCommand() {
		return PerforceDomain.FSTAT.getCommand();
	    }

	    @Override
	    public String[] getCommandArgs() {
		return new String[] {
			(allRevisions) ? allRevsFlags : oneRevFlags,
			path.asString(EnumSet.of(PathStringFormat.ESCAPE)) };
	    }
	});

    }

    public static Map<String, Object>[] getFstats(List<PerforcePath> paths,
	    Server server) throws ServerConnectionException,
	    ServerAccessException, ServerErrorException {
	// grab the fstat from each file revision
	final List<String> args = new ArrayList<String>();

	args.add(oneRevFlags);
	for (PerforcePath path : paths) {
	    args.add(path.asString(EnumSet.of(PathStringFormat.ESCAPE)));
	}

	return server.execute(new RequestImpl() {

	    @Override
	    public String getCommand() {
		return PerforceDomain.FSTAT.getCommand();
	    }

	    @Override
	    public String[] getCommandArgs() {
		return args.toArray(new String[args.size()]);
	    }
	});
    }
}
